@@ -41,13 +41,13 @@
  *****************************************************************************/
 
 /* Program version and copyright message */
-#define VERSION_MESSAGE     "vlc 0.2.71 Ourumov " \
+#define VERSION_MESSAGE     "vlc 0.2.81 Ourumov " \
                           /* "(" PROGRAM_BUILD ") (" PROGRAM_OPTIONS ")\n" */ \
                             "Copyright 1996-2001 VideoLAN\n"
-#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.71" \
+#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.81" \
                             " Ourumov - (C)1996-2001 VideoLAN\n"
 
-#define VERSION             "0.2.71"
+#define VERSION             "0.2.81"
 
 /*****************************************************************************
  * General compilation options
@@ -66,23 +66,23 @@
 /*****************************************************************************
  * Debugging options - define or undefine symbols
  *****************************************************************************/
-#ifdef DEBUG
-/* General debugging support, which depends of the DEBUG define, is determined
+#ifdef TRACE
+/* General trace support, which depends of the TRACE define, is determined
  * in the Makefile */
 
 /* Modules specific debugging - this will produce a lot of output, but can be
  * useful to track a bug */
-//#define DEBUG_INTF
-//#define DEBUG_INPUT
-//#define DEBUG_AUDIO
-#define DEBUG_VOUT
-#define DEBUG_VPAR
+//#define TRACE_INTF
+//#define TRACE_INPUT
+//#define TRACE_AUDIO
+#define TRACE_VOUT
+#define TRACE_VPAR
 
-/* Debugging log file - if defined, a file can be used to store all messages. If
- * DEBUG_LOG_ONLY is defined, debug messages will only be printed to the log and
+/* Trace log file - if defined, a file can be used to store all messages. If
+ * TRACE_LOG_ONLY is defined, debug messages will only be printed to the log and
  * will not appear on the screen */
-#define DEBUG_LOG                       "vlc-debug.log"
-#define DEBUG_LOG_ONLY
+#define TRACE_LOG                       "vlc-trace.log"
+#define TRACE_LOG_ONLY
 
 #endif
 
@@ -123,6 +123,7 @@
 
 /* Environment variable containing the display method */
 #define INTF_METHOD_VAR                 "vlc_intf"
+#define INTF_METHOD_DEFAULT             "gtk"
 
 /* Environment variable used to store startup script name and default value */
 #define INTF_INIT_SCRIPT_VAR            "vlcrc"
@@ -142,12 +143,21 @@
 /* Maximum number of channels */
 #define INTF_MAX_CHANNELS               10
 
+/* Default search path for interface file browser */
+#define INTF_PATH_VAR                   "vlc_search_path"
+#define INTF_PATH_DEFAULT               ""
+
+/* Environment variable containing the standard output method */
+#define INTF_STDOUT_VAR                 "vlc_stdout"
+#define INTF_STDOUT_DEFAULT             ""
+
 /*****************************************************************************
  * Input thread configuration
  *****************************************************************************/
 
-/* Environment variable containing the display method */
+/* Environment variable containing the input method */
 #define INPUT_METHOD_VAR                "vlc_input"
+#define INPUT_METHOD_DEFAULT            "ps"
 
 /* XXX?? */
 #define INPUT_IDLE_SLEEP                ((int)(0.100*CLOCK_FREQ))
@@ -169,19 +179,30 @@
 /* Maximum length of a hostname or source name */
 #define INPUT_MAX_SOURCE_LENGTH         100
 
+/* Default network protocol */
+#define INPUT_NETWORK_PROTOCOL_VAR      "vlc_network_protocol"
+#define INPUT_NETWORK_PROTOCOL_DEFAULT  "ts"
+
 /* Default remote server */
 #define INPUT_SERVER_VAR                "vlc_server"
-#define INPUT_SERVER_DEFAULT            "138.195.143.220"
-#define INPUT_BCAST_ADDR                "138.195.143.255"
+#define INPUT_SERVER_DEFAULT            "138.195.143.224"
 
 /* Default input port */
 #define INPUT_PORT_VAR                  "vlc_server_port"
 #define INPUT_PORT_DEFAULT              1234
 
+/* Default broadcast address */
+#define INPUT_BCAST_ADDR_VAR            "vlc_broadcast_addr"
+#define INPUT_BCAST_ADDR_DEFAULT        "138.195.143.255"
+
 /* Broadcast mode */
 #define INPUT_BROADCAST_VAR             "vlc_broadcast"
 #define INPUT_BROADCAST_DEFAULT         0
 
+/* Channels mode */
+#define INPUT_NETWORK_CHANNEL_VAR       "vlc_channel"
+#define INPUT_NETWORK_CHANNEL_DEFAULT   0
+
 /*
  * Channel method
  */
@@ -192,7 +213,7 @@
 
 /* Default server and port */
 #define INPUT_CHANNEL_SERVER_VAR           "vlc_channel_server"
-#define INPUT_CHANNEL_SERVER_DEFAULT       "138.195.139.95"
+#define INPUT_CHANNEL_SERVER_DEFAULT       "138.195.143.220"
 #define INPUT_CHANNEL_PORT_VAR             "vlc_channel_port"
 #define INPUT_CHANNEL_PORT_DEFAULT         6010
 
@@ -204,21 +225,31 @@
  * mark it to be presented */
 #define DEFAULT_PTS_DELAY               (mtime_t)(.2*CLOCK_FREQ)
 
+/* DVD defaults */
 #define INPUT_DVD_DEVICE_VAR            "vlc_dvd_device"
 #define INPUT_DVD_DEVICE_DEFAULT        "/dev/dvd"
 
 #define INPUT_TITLE_VAR                 "vlc_input_title"
 #define INPUT_CHAPTER_VAR               "vlc_input_chapter"
+#define INPUT_ANGLE_VAR                 "vlc_input_angle"
 #define INPUT_AUDIO_VAR                 "vlc_input_audio"
 #define INPUT_CHANNEL_VAR               "vlc_input_channel"
 #define INPUT_SUBTITLE_VAR              "vlc_input_subtitle"
 
+/* VCD defaults */
+#define INPUT_VCD_DEVICE_VAR            "vlc_vcd_device"
+#define INPUT_VCD_DEVICE_DEFAULT        "/dev/cdrom"
+
 /*****************************************************************************
  * Audio configuration
  *****************************************************************************/
 
+/* Maximum number of audio output threads */
+#define AOUT_MAX_THREADS                10
+
 /* Environment variable containing the audio output method */
 #define AOUT_METHOD_VAR                 "vlc_aout"
+#define AOUT_METHOD_DEFAULT             "dsp"
 
 /* Environment variable used to store dsp device name, and default value */
 #define AOUT_DSP_VAR                    "vlc_dsp"
@@ -237,6 +268,16 @@
 #define AOUT_STEREO_VAR                 "vlc_stereo"
 #define AOUT_STEREO_DEFAULT             1
 
+/* Environment variable for spdif mode */
+#define AOUT_SPDIF_VAR                  "vlc_spdif"
+#define AOUT_SPDIF_DEFAULT              0
+
+/* Environment variable containing the AC3 downmix method */
+#define DOWNMIX_METHOD_VAR              "vlc_downmix"
+
+/* Environment variable containing the AC3 IMDCT method */
+#define IMDCT_METHOD_VAR                "vlc_imdct"
+
 /* Volume */
 #define VOLUME_DEFAULT                  512
 #define VOLUME_STEP                     128
@@ -267,12 +308,16 @@
  * Video configuration
  *****************************************************************************/
 
+/* Maximum number of video output threads */
+#define VOUT_MAX_THREADS                10
+
 /*
  * Default settings for video output threads
  */
 
 /* Environment variable containing the display method */
 #define VOUT_METHOD_VAR                 "vlc_vout"
+#define VOUT_METHOD_DEFAULT             "x11"
 
 /* Environment variable containing the motion compensation method */
 #define MOTION_METHOD_VAR               "vlc_motion"
@@ -293,6 +338,15 @@
 #define VOUT_WIDTH_DEFAULT              720
 #define VOUT_HEIGHT_DEFAULT             576
 
+#define VOUT_DEPTH_VAR                  "vlc_depth"
+#define VOUT_DEPTH_DEFAULT              15
+
+#define VOUT_FULLSCREEN_VAR             "vlc_fullscreen"
+#define VOUT_FULLSCREEN_DEFAULT         0
+
+#define VOUT_FULLSCREEN_DEPTH_VAR       "vlc_fullscreen_depth"
+#define VOUT_FULLSCREEN_DEPTH_DEFAULT   32
+
 /* Maximum width of a scaled source picture - this should be relatively high,
  * since higher stream values will result in no display at all. */
 #define VOUT_MAX_WIDTH                  4096
@@ -324,7 +378,8 @@
 #define VOUT_OVERLAY_DEFAULT            0
 
 /* Default gamma */
-#define VOUT_GAMMA                      0.
+#define VOUT_GAMMA_VAR                  "vlc_gamma"
+#define VOUT_GAMMA_DEFAULT              0.
 
 /* Default fonts */
 #define VOUT_DEFAULT_FONT               "default8x9.psf"
@@ -366,7 +421,7 @@
 #define VOUT_FB_DEV_DEFAULT             "/dev/fb0"
 
 /* The default video output window title */
-#define VOUT_TITLE                      "VideoLAN Client 0.2.71"
+#define VOUT_TITLE                      "VideoLAN Client 0.2.81"
 
 /*****************************************************************************
  * Video parser configuration
@@ -384,13 +439,6 @@
  * that raising this level dramatically lengthens the compilation time. */
 #define VPAR_OPTIM_LEVEL                2
 
-/* The following directives only apply if you define VDEC_SMP below. */
-
-/* Number of macroblock buffers available. It should be always greater than
- * twice the number of macroblocks in a picture. VFIFO_SIZE + 1 should also
- * be a power of two. */
-#define VFIFO_SIZE                      8191
-
 /* Maximum number of macroblocks in a picture. */
 #define MAX_MB                          2048
 
@@ -401,27 +449,20 @@
  * Video decoder configuration
  *****************************************************************************/
 
-//#define VDEC_SMP
-
 #define VDEC_IDLE_SLEEP                 ((int)(0.100*CLOCK_FREQ))
 
-/* Number of video_decoder threads to launch on startup of the video_parser.
- * It should always be less than half the number of macroblocks of a
- * picture. Only available if you defined VDEC_SMP above. */
-#define NB_VDEC                         1
-
 /* Maximum range of values out of the IDCT + motion compensation. */
 #define VDEC_CROPRANGE                  2048
 
+/* Environment variable containing the SMP value. */
+#define VDEC_SMP_VAR                   "vlc_smp"
+
+/* No SMP by default, since it slows down things on non-smp machines. */
+#define VDEC_SMP_DEFAULT                0
+
 /* Nice increments for decoders -- necessary for x11 scheduling */
 #define VDEC_NICE                       3
 
-/*****************************************************************************
- * Generic decoder configuration
- *****************************************************************************/
-
-#define GDEC_IDLE_SLEEP                 ((int)(0.100*CLOCK_FREQ))
-
 /*****************************************************************************
  * Messages and console interfaces configuration
  *****************************************************************************/
@@ -435,14 +476,14 @@
 #define INTF_MSG_QSIZE                  64
 
 /* Interface warnig message level */
-#define INTF_WARNING_VAR                "warning_level"
-#define INTF_WARNING_DEFAULT            12
+#define INTF_WARNING_VAR                "vlc_warning_level"
+#define INTF_WARNING_DEFAULT            0
 
 /* Define to enable messages queues - disabling messages queue can be useful
  * when debugging, since it allows messages which would not be printed
  * due to a crash to be printed anyway */
 #ifndef DEBUG
-//#define INTF_MSG_QUEUE
+#define INTF_MSG_QUEUE
 #endif
 
 /* Format of the header for debug messages. The arguments following this header
@@ -450,7 +491,7 @@
  * message function was called */
 #define INTF_MSG_DBG_FORMAT             "## %s:%s(),%i: "
 
-/* Maximal number of arguments on a command line, including the function name */
+/* Max number of arguments on a command line, including the function name */
 #define INTF_MAX_ARGS                   20
 
 /* Maximal size of a command line in a script */
@@ -462,3 +503,19 @@
 /* Maximal number of commands which can be saved in history list */
 #define INTF_CONSOLE_MAX_HISTORY        20
 
+/****************************************************************************
+ * Playlist defaults
+ ****************************************************************************/
+
+/* Launch on start-up */
+#define PLAYLIST_STARTUP_VAR            "vlc_playlist_on_start_up"
+#define PLAYLIST_STARTUP_DEFAULT        0
+
+/* Enqueue drag'n dropped item */
+#define PLAYLIST_ENQUEUE_VAR            "vlc_playlist_enqueue"
+#define PLAYLIST_ENQUEUE_DEFAULT        0
+
+/* Loop on playlist end */
+#define PLAYLIST_LOOP_VAR               "vlc_playlist_loop"
+#define PLAYLIST_LOOP_DEFAULT           0
+
