@@ -4,7 +4,7 @@
  * decoders.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input.c,v 1.118 2001/05/31 12:45:39 sam Exp $
+ * $Id: input.c,v 1.127 2001/07/18 14:21:00 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -74,8 +74,8 @@
 #include "stream_control.h"
 #include "input_ext-intf.h"
 #include "input_ext-dec.h"
+#include "input_ext-plugins.h"
 
-#include "input.h"
 #include "interface.h"
 
 #include "main.h"
@@ -138,6 +138,7 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->stream.i_selected_es_number = 0;
     p_input->stream.i_pgrm_number = 0;
     p_input->stream.i_new_status = p_input->stream.i_new_rate = 0;
+    p_input->stream.b_new_mute = MUTE_NO_CHANGE;
     p_input->stream.i_mux_rate = 0;
 
     /* no stream, no area */
@@ -145,7 +146,8 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->stream.pp_areas = NULL;
     p_input->stream.p_selected_area = NULL;
     p_input->stream.p_new_area = NULL;
-    /* By default there is one areas in a stream */
+
+    /* By default there is one area in a stream */
     input_AddArea( p_input );
     p_input->stream.p_selected_area = p_input->stream.pp_areas[0];
 
@@ -153,7 +155,10 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->stream.control.i_status = PLAYING_S;
     p_input->stream.control.i_rate = DEFAULT_RATE;
     p_input->stream.control.b_mute = 0;
-    p_input->stream.control.b_bw = 0;
+    p_input->stream.control.b_grayscale = main_GetIntVariable(
+                            VOUT_GRAYSCALE_VAR, VOUT_GRAYSCALE_DEFAULT );
+    p_input->stream.control.i_smp = main_GetIntVariable(
+                            VDEC_SMP_VAR, VDEC_SMP_DEFAULT );
 
     /* Setup callbacks */
     p_input->pf_file_open     = FileOpen;
@@ -163,10 +168,7 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->pf_network_close = NetworkClose;
 #endif
 
-    /* Create thread and set locks. */
-    vlc_mutex_init( &p_input->stream.stream_lock );
-    vlc_cond_init( &p_input->stream.stream_wait );
-    vlc_mutex_init( &p_input->stream.control.control_lock );
+    /* Create thread. */
     if( vlc_thread_create( &p_input->thread_id, "input", (void *) RunThread,
                            (void *) p_input ) )
     {
@@ -242,6 +244,7 @@ static void RunThread( input_thread_t *p_input )
         p_input->b_error = 1;
         ErrorThread( p_input );
         DestroyThread( p_input );
+        free( p_input );
         return;
     }
 
@@ -269,7 +272,23 @@ static void RunThread( input_thread_t *p_input )
 
         if( p_input->stream.p_new_area )
         {
-            p_input->pf_set_area( p_input, p_input->stream.p_new_area );
+            if( p_input->stream.b_seekable && p_input->pf_set_area != NULL )
+            {
+
+                p_input->pf_set_area( p_input, p_input->stream.p_new_area );
+
+                for( i = 0; i < p_input->stream.i_pgrm_number; i++ )
+                {
+                    pgrm_descriptor_t * p_pgrm
+                                            = p_input->stream.pp_programs[i];
+                    /* Escape all decoders for the stream discontinuity they
+                     * will encounter. */
+                    input_EscapeDiscontinuity( p_input, p_pgrm );
+
+                    /* Reinitialize synchro. */
+                    p_pgrm->i_synchro_state = SYNCHRO_REINIT;
+                }
+            }
             p_input->stream.p_new_area = NULL;
         }
 
@@ -307,6 +326,20 @@ static void RunThread( input_thread_t *p_input )
             p_input->stream.p_newly_selected_es = NULL;
         }
 
+        if( p_input->stream.b_new_mute != MUTE_NO_CHANGE )
+        {
+            if( p_input->stream.b_new_mute )
+            {
+                input_EscapeAudioDiscontinuity( p_input );
+            }
+
+            vlc_mutex_lock( &p_input->stream.control.control_lock );
+            p_input->stream.control.b_mute = p_input->stream.b_new_mute;
+            vlc_mutex_unlock( &p_input->stream.control.control_lock );
+
+            p_input->stream.b_new_mute = MUTE_NO_CHANGE;
+        }
+
         vlc_mutex_unlock( &p_input->stream.stream_lock );
 
         i_error = p_input->pf_read( p_input, pp_packets );
@@ -362,6 +395,11 @@ static int InitThread( input_thread_t * p_input )
     p_input->c_packets_trashed          = 0;
 #endif
 
+    /* Set locks. */
+    vlc_mutex_init( &p_input->stream.stream_lock );
+    vlc_cond_init( &p_input->stream.stream_wait );
+    vlc_mutex_init( &p_input->stream.control.control_lock );
+
     /* Default, might get overwritten */
     p_input->pf_open = p_input->pf_file_open;
     p_input->pf_close = p_input->pf_file_close;
@@ -387,6 +425,7 @@ static int InitThread( input_thread_t * p_input )
         p_input->pf_close         = f.pf_close;
     }
     p_input->pf_end           = f.pf_end;
+    p_input->pf_init_bit_stream= f.pf_init_bit_stream;
     p_input->pf_read          = f.pf_read;
     p_input->pf_set_area      = f.pf_set_area;
     p_input->pf_demux         = f.pf_demux;
@@ -397,12 +436,13 @@ static int InitThread( input_thread_t * p_input )
     p_input->pf_rewind        = f.pf_rewind;
     p_input->pf_seek          = f.pf_seek;
 #undef f
+
+    /* We found the appropriate plugin, open the target */
     p_input->pf_open( p_input );
 
     if( p_input->b_error )
     {
         /* We barfed -- exit nicely */
-        p_input->pf_close( p_input );
         module_Unneed( p_input->p_input_module );
         return( -1 );
     }
@@ -500,10 +540,6 @@ static void FileOpen( input_thread_t * p_input )
     struct stat         stat_info;
     int                 i_stat;
 
-#if defined( WIN32 )
-    char buf[7] = { 0 };
-#endif
-
     char *psz_name = p_input->p_source;
 
     /* FIXME: this code ought to be in the plugin so that code can
@@ -518,9 +554,6 @@ static void FileOpen( input_thread_t * p_input )
             /* get rid of the 'dvd:' stuff and try again */
             psz_name += 4;
             i_stat = stat( psz_name, &stat_info );
-#if defined( WIN32 )
-            _snprintf( buf, 7, "\\\\.\\%c:", psz_name[0] );
-#endif
         }
         else if( ( i_size > 5 )
                  && !strncasecmp( psz_name, "file:", 5 ) )
@@ -530,11 +563,7 @@ static void FileOpen( input_thread_t * p_input )
             i_stat = stat( psz_name, &stat_info );
         }
 
-        if( i_stat == (-1) 
-#if defined( WIN32 )
-        && !buf[0]      
-#endif
-            )
+        if( i_stat == (-1) )
         {
             intf_ErrMsg( "input error: cannot stat() file `%s' (%s)",
                          psz_name, strerror(errno));
@@ -549,11 +578,7 @@ static void FileOpen( input_thread_t * p_input )
     p_input->stream.b_pace_control = 1;
 
     if( S_ISREG(stat_info.st_mode) || S_ISCHR(stat_info.st_mode)
-         || S_ISBLK(stat_info.st_mode)
-#if defined( WIN32 )
-         || ( buf[0] && ( ( stat_info.st_size = 0 ) == 0 ) )
-#endif
-         )
+         || S_ISBLK(stat_info.st_mode) )
     {
         p_input->stream.b_seekable = 1;
         p_input->stream.p_selected_area->i_size = stat_info.st_size;
@@ -580,14 +605,11 @@ static void FileOpen( input_thread_t * p_input )
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
     intf_WarnMsg( 1, "input: opening file `%s'", p_input->p_source );
-#ifndef WIN32
+#if defined( WIN32 )
+    if( (p_input->i_handle = open( psz_name, O_BINARY ) ) == (-1) )
+#else
     if( (p_input->i_handle = open( psz_name,
                                    /*O_NONBLOCK | O_LARGEFILE*/0 )) == (-1) )
-#else
-    if( ( buf[0] && ( (HANDLE) p_input->i_handle = CreateFile( buf, 
-        GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, 
-        NULL, OPEN_EXISTING, 0, NULL ) ) == INVALID_HANDLE_VALUE ) ||
-        ( !buf[0] && (p_input->i_handle = open( psz_name, O_BINARY ) ) == (-1) ) )
 #endif
     {
         intf_ErrMsg( "input error: cannot open file (%s)", strerror(errno) );
@@ -603,14 +625,7 @@ static void FileOpen( input_thread_t * p_input )
 static void FileClose( input_thread_t * p_input )
 {
     intf_WarnMsg( 1, "input: closing file `%s'", p_input->p_source );
-#if defined( WIN32 )
-    if( ( strlen( p_input->p_source ) > 4 ) &&
-        !strncasecmp( p_input->p_source, "dvd:", 4 ) )
-    {
-        CloseHandle( (HANDLE) p_input->i_handle );
-    }
-    else
-#endif
+
     close( p_input->i_handle );
 
     return;
@@ -627,11 +642,14 @@ static void NetworkOpen( input_thread_t * p_input )
     int                 i_port = 0;
     int                 i_opt;
     struct sockaddr_in  sock;
-
+#ifdef WIN32
+    WSADATA Data;
+    int i_err;
+#endif
+    
 #ifdef WIN32
     /* WinSock Library Init. */
-    WSADATA Data;
-    int i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
+    i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
 
     if( i_err )
     {
@@ -782,6 +800,13 @@ static void NetworkOpen( input_thread_t * p_input )
         p_input->b_error = 1;
         return;
     }
+
+#if defined( WIN32 )
+    if ( psz_broadcast != NULL )
+    {
+        sock.sin_addr.s_addr = INADDR_ANY;
+    }
+#endif
     
     /* Bind it */
     if( bind( p_input->i_handle, (struct sockaddr *)&sock, 
